# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>

   # include "command.h"
   # include "color_theme.h"
   # include "command_line.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "datasystem.h"
   # include "entry.h"
   # include "history.h"
   # include "key.h"
   # include "mycurses.h"
   # include "message.h"
   # include "screen.h"
   # include "status_line.h"
   # include "string.h"
# endif
# if GLOBAL
   char * _entry_line;
   char * _entry_line_keep;

   char * _command_line;

   int command_line_stop = 0;
   int command_line_size;
   int command_line_memsize;
   int entry_line_memsize = 82;
   int entry_line_size;

   static int cursor_y;
   static int cursor_x;

   char * _confirm_str[] = 
      { "Yes",
        "No",
      };
   char * _search_rstr[] = 
      { "Yes",
        "No",
        "All",
      };
   char * _set_str[] =
      { "color theme",
        "syntax",
        "tab",
        "line feed"
      };
   char * _color_theme_str[] =
      { "Standard",
        "New",
        "Midnight",
        "Blue"
      };
   char * _syntax_str[] =
      { "None",
        "C/C++",
      };
   char * _lf_str[] =
      { "Normal",
        "Smart",
      };

   int set_cardinal = sizeof(_set_str)/sizeof(char *);
   int syntax_cardinal = sizeof(_syntax_str)/sizeof(char *);
   int lf_cardinal = sizeof(_lf_str)/sizeof(char *);
# endif
# if PROTO
   void command_line_read(History * _history);
   void command_line_redraw();
# endif

void entry_line_init()
   { _entry_line      = (char *) calloc(1, entry_line_memsize);
     _entry_line_keep = (char *) calloc(1, entry_line_memsize);

     return;
   }
void command_line_init()
   { command_line_memsize = screen_cols;

     _command_line = (char *) malloc(command_line_memsize);
     _memset(_command_line, ' ', command_line_memsize);

      return;
   }

void command_line_echo(char * _str)
   { int len = string_len(_str);
     if ( len < command_line_memsize - 1 )
        { string_copy(_command_line, _str);
          command_line_size = len;
          command_line_stop = len;
          command_line_redraw();
        }
     return;
   }

void command_line_print(char * _template, ...)
   { va_list ap;
     va_start(ap, _template);
     vsprintf(_entry_line, _template, ap);

     entry_line_size = string_len(_entry_line);
     if ( entry_line_size < command_line_memsize )
        { string_copy(_command_line, _entry_line);
        }
     return;
   }
void command_line_stop_print(char * _template, ...)
   { command_line_back_to_stop();

     va_list ap;
     va_start(ap, _template);
     vsprintf(_entry_line, _template, ap);

     entry_line_size = string_len(_entry_line);
     if ( command_line_stop + entry_line_size < command_line_memsize )
        { string_copy(_command_line + command_line_stop, _entry_line);
          command_line_size = command_line_stop + entry_line_size;
          cursor_x = command_line_stop + entry_line_size;
        }
     else
        { //status_line_print("Entry too long");
          int delta = command_line_stop + entry_line_size - command_line_memsize + 1;
          string_copy(_command_line + command_line_stop, _entry_line + delta);
          command_line_size = command_line_memsize - 1;
          cursor_x = command_line_memsize - 1;
        }
     //int len = string_len(_command_line);
     //command_line_size = len;
     //cursor_x = len;
     command_line_redraw();

     return;
   }

void command_line_redraw()
   { curses_mvaddnstr(COMMAND_LINE, 0, _command_line, screen_cols);
     cursor_move(cursor_y, cursor_x);

     return;
   }

void command_line_clear()
   { _memset(_command_line, ' ', command_line_memsize);
     command_line_stop = 0;

     return;
   }
void command_line_back_to_stop()
   { char * _command_line_point = _command_line + command_line_stop;
     int cols = command_line_memsize - command_line_stop;

     for ( int i = 0; i < cols; i++ )
        { * _command_line_point++ = ' ';
        }
     cursor_x = command_line_stop;
     command_line_redraw();
     cursor_move(cursor_y, cursor_x);

     return;
   }
void command_line_backspace()
   { if ( command_line_size == command_line_stop )
        { return;
        }
     * (_command_line + --command_line_size) = ' ';
     cursor_x--;

     return;
   }

void command_line_enter()
   { command_line_echo(_message[3]);

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;

     command_line_read(_history_ptr[HISTORY_COMMAND]);
     if ( * _entry_line )
        { int n = command_recognize();
          if ( n >= 0 )
             { command_execute(n);
             }
        }
     command_line_clear();
     command_line_redraw();
     cursor_move(* _cursor_y, * _cursor_x);

     return;
   }

void command_line_read(History * _history)
   { char * _command_line_point = _command_line + command_line_stop;
     * _entry_line = 0;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;
     bool history_match = 0;

     do
        { command_line_redraw();

          * _command_line_point = 0;
          string_copy(_entry_line, _command_line + command_line_stop);

          c = curses_getch();

          if ( c == KEY_UP || c == CTRL_P )
             { string_copy(_entry_line_keep, _entry_line);
               c = history_complete(_history);
               if ( c == CTRL_A )
                  { command_line_stop_print("%s", _entry_line_keep);
                  }
               _command_line_point = _command_line + command_line_size;
               status_line_refresh();
             }
          if ( c == CTRL_I )
             { if ( _history == _history_ptr[HISTORY_FILE_OPEN] || _history == _history_ptr[HISTORY_FILE_SAVE] )
                  { string_copy(_entry_line_keep, _entry_line);
                    c = entry_complete();
                    if ( c == CTRL_A )
                       { command_line_stop_print("%s", _entry_line_keep);
                       }
                    _command_line_point = _command_line + command_line_size;
                    status_line_refresh();
                  }
             }
          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               continue;
             }
          if ( c > ' ' && c < 256 )
             { if ( command_line_size < command_line_memsize - 1 )
                  { command_line_size++;
                    * _command_line_point++ = c;
                    cursor_x++;
                  }
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( c )
        { if ( * _entry_line )
             { history_update(_history);
             }
        }
     else
        { * _entry_line = 0;
        }
     
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }
void command_line_read_number()
   { char * _command_line_point = _command_line + command_line_stop;
     * _entry_line = 0;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;

     do
        { command_line_redraw();

          * _command_line_point = 0;
          string_copy(_entry_line, _command_line + command_line_stop);

          c = curses_getch();

          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               continue;
             }
          if ( c >= '0' && c <= '9' )
             { if ( command_line_size < command_line_memsize - 1 )
                  { command_line_size++;
                    * _command_line_point++ = c;
                    cursor_x++;
                  }
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( c )
        { * _command_line_point = 0;
          string_copy(_entry_line, _command_line + command_line_stop);
        }
     else
        { * _entry_line = 0;
        }
 
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }
void command_line_read_lock()
   { char * _command_line_point = _command_line + command_line_stop;
     char * _user_line_point = _entry_line;

     cursor_y = COMMAND_LINE;
     cursor_x = command_line_stop;
      command_line_size = command_line_stop;
     int c;

     do
        { command_line_redraw();
          c = curses_getch();
          if ( c == KEY_BACKSPACE || c == CTRL_H )
             { if ( command_line_size == command_line_stop )
                  { continue;
                  }
                command_line_backspace();
               _command_line_point--;
               _user_line_point--;
               continue;
             }
          if ( c >= ' ' && c < 256 )
             { if ( command_line_size < command_line_memsize - 1 )
                  { command_line_size++;
                    * _command_line_point++ = '*';
                    * _user_line_point++ = c;
                    cursor_x++;
                  }
             }
        }
     while ( c && c != CTRL_J && c != CTRL_M );

     if ( ! c )
        { _command_line_point = _command_line;
          _user_line_point = _entry_line;
          * _user_line_point = 0;
        }
     else
        { * _command_line_point = 0;
          * _user_line_point = 0;
          //string_copy(_entry_line, _command_line + command_line_stop);
        }
 
     command_line_clear();
     command_line_redraw();

     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }

void command_line_revolve_echo(char * _str)
   { command_line_back_to_stop();

     int len = string_len(_str);
     if ( command_line_stop + len < command_line_memsize - 1 )
        { string_copy(_command_line + command_line_stop, _str);
          command_line_size = len;
          command_line_redraw();
        }
     return;
   }
int  command_line_revolve(char * _revolve_str[], int start, int n)
   { cursor_hide();

     int k = start;
     int c;
     command_line_stop = command_line_size;
     command_line_revolve_echo(_revolve_str[start]);
     do
       { c = curses_getch();
         switch (c)
            { case KEY_UP:
              case CTRL_P:
                 if ( k )
                    { k--;
                      //command_line_back_to_stop();
                      command_line_revolve_echo(_revolve_str[k]);
                    }
                 break;
              case KEY_DOWN:
              case CTRL_N:
                 if ( k < n - 1 )
                    { k++;
                      //command_line_back_to_stop();
                      command_line_revolve_echo(_revolve_str[k]);
                    }
                 break;
              case CTRL_M:
              case CTRL_J:
                 c = 0;
                 break;
              case 0:
                 k = REPLY_CANCEL;
                 break;
            }
       }
    while ( c );

    cursor_show();
    command_line_clear();
    command_line_redraw();
    if ( datasystem_count )
       { cursor_move(* _cursor_y, * _cursor_x);
       }
    return k;
   }
