# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>

   # include "block.h"
   # include "cwrite.h"
   # include "datasystem.h"
   # include "message.h"
   # include "mycurses.h"
   # include "screen.h"
   # include "scroll.h"
   # include "status_line.h"
# endif
# if CONST
   # define COLLAPSE_FLAG  1
   # define EXPAND_FLAG   -1
   # define DETACH_FLAG    1
   # define ATTACH_FLAG   -1
# endif
# if GLOBAL
   Datachain ** __datachain_detach_start;
   Datachain ** __datachain_detach_end;
# endif

void block_toggle_view(Datachain * _datachain_block_begin, Datachain * _datachain_block_end, int flag)
   { int sid = 101;

     Datachain * _datachain = _datachain_block_begin;
     do
        { _datachain->invisible += flag;
          _datachain = _datachain->_next;
        }
     while ( _datachain != _datachain_block_end->_next );
     //while ( (_datachain != _datachain_block_end) && (_datachain = _datachain->_next) );
     # ifdef TEST_BLOCK_TOGGLE_VIEW
        stop(sid, "here");
        if ( _datachain )
           { printf("%d\n", _datachain_block_begin->ordinal);
             printf("%d\n", _datachain_block_end->ordinal);
           }
        exit(-1);
     # endif
     screen_redraw();

     return;
   }
void block_toggle_state(int flag)
   { Datachain * _datachain = * __datachain_first;

     while ( _datachain != * __datachain_detach_start )
        { _datachain->invisible += flag;
          _datachain = _datachain->_next;
        }
     _datachain = (* __datachain_detach_end)->_next;
     while ( _datachain )
        { _datachain->invisible += flag;
          _datachain = _datachain->_next;
        }
  
     return;
   }

void block_collapse(Datachain * _datachain_block_begin, Datachain * _datachain_block_end)
   { block_toggle_view(_datachain_block_begin, _datachain_block_end, COLLAPSE_FLAG);

     return;
   }
void block_expand(Datachain * _datachain_block_begin, Datachain * _datachain_block_end)
   { block_toggle_view(_datachain_block_begin, _datachain_block_end, EXPAND_FLAG);

     return;
   }

void view_collapse()
   { Datachain * _datachain_block_begin, * _datachain_block_end;
     Datachain * _datachain, * _datachain_end;
     int block_level;

     if ( * _status_register & DETACH_STATE )
         { _datachain = * __datachain_detach_start;
           _datachain_end = * __datachain_detach_end;
         }
     else
        { _datachain = * __datachain_first;
          _datachain_end = * __datachain_last;
        }
     while ( _datachain )
     //do
        { block_level = block_locate(_datachain, & _datachain_block_begin, & _datachain_block_end);
          //if ( block_level > 0 )
          if ( block_level > 0 && ! _datachain_block_begin->invisible )
             { block_toggle_view(_datachain_block_begin, _datachain_block_end, COLLAPSE_FLAG);
                  // -- move to block header --
               if ( (* __datachain)->invisible )
                  { * __datachain = _datachain; 
                    * _dataline_offset = 0;
                    * _cursor_x = screen_left;
                  }
               _datachain = _datachain_block_end->_next;
             }
          else
             { _datachain = _datachain->_next;
             }
        }
     //while ( _datachain != _datachain_end);

     int n = visible_lines_up(* __datachain);
     if ( screen_top + n - 1 < * _cursor_y )
        { * _cursor_y = screen_top + n - 1;
        }
     screen_redraw();

     return;
   }
void view_expand()
   { Datachain * _datachain = * __datachain_first;

     while ( _datachain )
        { _datachain->invisible = 0;
          _datachain = _datachain->_next;
        }
     screen_redraw();

     return;
   }

void block_detach()
   { block_toggle_state(DETACH_FLAG);
     _datasystem->cursor_y_detach = * _cursor_y;
     * _status_register |= DETACH_STATE;
     * _dataline_offset = 0;
     * _cursor_y = screen_top;
     * _cursor_x = screen_left;
     screen_redraw();
     return;
   }
void block_attach()
   { block_toggle_state(ATTACH_FLAG);
     * _status_register &= ~DETACH_STATE;
     * __datachain = * __datachain_detach_start;
     * _dataline_offset = 0;
     * _cursor_y = _datasystem->cursor_y_detach;
     * _cursor_x = screen_left;
     screen_redraw();

     return;
   }

void block_collapse_max(Datachain * _datachain_header)
   { Datachain * _datachain_block_begin, * _datachain_block_end;
     Datachain * _datachain_block_begin_2, * _datachain_block_end_2;

     int block_level = block_locate(_datachain_header, & _datachain_block_begin, & _datachain_block_end);
     if ( block_level <= 0 )
        { return;
        }
     block_toggle_view(_datachain_block_begin, _datachain_block_end, COLLAPSE_FLAG);

     Datachain * _datachain = _datachain_block_begin;
     do
        { block_level = block_locate(_datachain, & _datachain_block_begin_2, & _datachain_block_end_2);
          if ( block_level > 0 )
             { block_toggle_view(_datachain_block_begin_2, _datachain_block_end_2, COLLAPSE_FLAG);
             }
        }
     while ( (_datachain != _datachain_block_end) && (_datachain = _datachain->_next) );

     return;
   }
void block_expand_max()
   { Datachain * _datachain_block_begin, * _datachain_block_end;

     int block_level = block_locate(* __datachain, & _datachain_block_begin, & _datachain_block_end);
     if ( block_level <= 0 )
        { return;
        }
     Datachain * _datachain = _datachain_block_begin;
     do
        { _datachain->invisible = 0;
        }
     while ( (_datachain != _datachain_block_end) && (_datachain = _datachain->_next) );
     screen_redraw();
     return;
   }

void view_collapse_max()
   { Datachain * _datachain_block_begin, * _datachain_block_end;
     int block_level;

     Datachain * _datachain = * __datachain_first;
     while ( _datachain )
        { block_level = block_locate(_datachain, & _datachain_block_begin, & _datachain_block_end);
          if ( block_level > 0 )
             { block_collapse_max(_datachain);
               _datachain = _datachain_block_end->_next;
             }
          else
             { _datachain = _datachain->_next;
             }
        }
     screen_redraw();
     return;
   }
